'use client'

import { Button } from '@/components/ui/button'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { useToast } from '@/components/ui/use-toast'
import axiosClient from '@/services/axiosClient'
import { zodResolver } from '@hookform/resolvers/zod'
import { AxiosError } from 'axios'
import { useState } from 'react'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

const formSchema = z
  .object({
    old_password: z.string().min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
    new_password: z.string().min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
    new_password_confirmation: z
      .string()
      .min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
  })
  .refine((data) => data.new_password === data.new_password_confirmation, {
    message: 'Գաղտնաբառը չի համընկնում',
    path: ['new_password_confirmation'],
  })

export default function ChangePassword() {
  const [loading, setLoading] = useState(false)
  const { toast } = useToast()
  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      old_password: '',
      new_password: '',
      new_password_confirmation: '',
    },
  })

  async function handlePasswordChange(values: z.infer<typeof formSchema>) {
    setLoading(true)
    try {
      await axiosClient.post('api/change-password', values)
      toast({
        title: 'Գաղտնաբառը փոխված է',
      })
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  return (
    <Form {...form}>
      <form
        onSubmit={form.handleSubmit(handlePasswordChange)}
        className='flex w-full max-w-xl flex-col items-center justify-center gap-6'
      >
        <h1 className='self-start text-2xl font-bold'>Փոխել գաղտնաբառը </h1>
        <FormField
          name='old_password'
          render={({ field }) => (
            <FormItem>
              <FormLabel required={true}>Հին գաղտնաբառ</FormLabel>
              <FormControl>
                <Input {...field} type='password' />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          name='new_password'
          render={({ field }) => (
            <FormItem>
              <FormLabel required={true}>Նոր գաղտնաբառ</FormLabel>
              <FormControl>
                <Input {...field} type='password' />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          name='new_password_confirmation'
          render={({ field }) => (
            <FormItem>
              <FormLabel required={true}>Հաստատեք գաղտնաբառը</FormLabel>
              <FormControl>
                <Input {...field} type='password' />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <Button className='w-full font-bold' disabled={loading}>
          Պահպանել
        </Button>
      </form>
    </Form>
  )
}
