'use client'

import { usePathname, useRouter } from 'next/navigation'
import { useEffect, useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import { Label } from '@radix-ui/react-label'
import { useMutation, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import PaymentMethods from '@/app/(checkout)/PaymentMethods'
import { Button } from '@/components/ui/button'
import { Checkbox } from '@/components/ui/checkbox'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import {
  Select,
  SelectContent,
  SelectGroup,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from '@/components/ui/select'
import { Textarea } from '@/components/ui/textarea'
import { useToast } from '@/components/ui/use-toast'
import { CREDIT_CARD_ID, IDRAM_ID } from '@/constants'
import { states } from '@/data/data'
import useGetAccountData from '@/hooks/server/useGetAccountData'
import useGetCheckoutData from '@/hooks/server/useGetCheckoutData'
import useGetShippingData from '@/hooks/server/useGetShippingData'
import {
  cleanObject,
  cn,
  convertToMoney,
  formatInputValue,
  handleIdram,
  slicePhoneCode,
} from '@/lib/utils'
import paymentSchema from '@/schemas/paymentSchema'
import profileSchema from '@/schemas/profileSchema'
import axiosClient from '@/services/axiosClient'
import { useCheckoutStore } from '@/store/checkout'

const ProfileAndPay = ({ isInPayment = false }: { isInPayment?: boolean }) => {
  const { toast } = useToast()
  const token = Cookies.get('siamoods_token')
  const pathname = usePathname()
  const router = useRouter()
  const schema = isInPayment ? paymentSchema : profileSchema
  const [paymentType, setPaymentType] = useState('1')
  const queryClient = useQueryClient()
  const setCheckoutStore = useCheckoutStore((state) => state.setCheckoutStore)
  const restoreCheckoutStore = useCheckoutStore(
    (state) => state.restoreCheckoutStore
  )
  const deliveryTypeId = useCheckoutStore((state) => state.deliveryTypeId)
  const gift_card = useCheckoutStore((state) => state.gift_card)
  const promotion = useCheckoutStore((state) => state.promotion)
  const { shippingData } = useGetShippingData()
  const currentShipping = shippingData?.find(
    (item) => item.shipping_type_id === +deliveryTypeId
  )
  const { checkoutData } = useGetCheckoutData()
  const { accountData } = useGetAccountData()

  useEffect(() => {
    if (accountData?.email) {
      setCheckoutStore('email', accountData?.email)
    }

    return () => {
      restoreCheckoutStore()
    }
  }, [])

  const mutation = useMutation({
    mutationFn: (values) => {
      if (isInPayment) {
        return axiosClient.post('api/orders', values)
        // return new Promise((resolve) => {
        //   setTimeout(() => {
        //     resolve({ data: { order_id: '10852' } })
        //   }, 1000)
        // })
      } else {
        return axiosClient.post('api/account', values)
      }
    },
    onSuccess: (data) => {
      if (!isInPayment) {
        queryClient.invalidateQueries({ queryKey: ['account'] })
        toast({
          title: 'Պահպանված է',
        })
      } else {
        const res = data.data
        if (paymentType === IDRAM_ID) {
          handleIdram({
            EDP_REC_ACCOUNT: res?.payment_info?.account,
            EDP_DESCRIPTION: res?.payment_info?.description,
            EDP_AMOUNT: res?.payment_info?.amount,
            EDP_BILL_NO: res?.payment_key,
          })
          localStorage.setItem('payment_method', IDRAM_ID)
          return
        }
        if (paymentType === CREDIT_CARD_ID) {
          window.open(
            res?.payment_info?.url,
            '_blank',
            'width=800,height=600,scrollbars=yes,resizable=yes'
          )
          localStorage.setItem('payment_method', CREDIT_CARD_ID)
          return
        }
        localStorage.setItem('totalPrice', res?.payment_info?.amount)
        localStorage.setItem(
          'cartItemsIds',
          JSON.stringify(checkoutData?.products.map((item) => item.haysell_id))
        )

        const orderId = res.order_id
        if (orderId) {
          queryClient.invalidateQueries({ queryKey: ['cart'] })
          queryClient.invalidateQueries({ queryKey: ['orders'] })
          setCheckoutStore('navigatedFromCheckout', '1')
          if (token) {
            // router.push(`/orders`)
            router.push(`/orders/${orderId}`)
          } else {
            router.push(`/guest-orders/${orderId}`)
          }
        }
      }
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  const form = useForm({
    resolver: zodResolver(schema),
    defaultValues: {
      email: accountData?.email || '',
      profileFirstName: accountData?.name || '',
      profileLastName: accountData?.lastname || '',
      profilePhone: slicePhoneCode(accountData?.phone),
      subscribe: accountData?.subscribe == 1 ? true : false,

      note: '',

      firstName: accountData?.shipping?.name || '',
      lastName: accountData?.shipping?.lastname || '',
      address: accountData?.shipping?.address_1 || '',
      cityOrVillage: accountData?.shipping?.city || '',
      state: accountData?.shipping?.state || 'ER',
      phone: slicePhoneCode(accountData?.shipping?.phone),
      zip: accountData?.shipping?.zip || '',

      payerAddressIfDifferent:
        accountData?.shipping?.same_for_payment === 0 ? true : false,

      payerFirstName: accountData?.payment?.name || '',
      payerLastName: accountData?.payment?.lastname || '',
      payerAddress: accountData?.payment?.address_1 || '',
      payerCityOrVillage: accountData?.payment?.city || '',
      payerState: accountData?.payment?.state || '',
      payerPhone: slicePhoneCode(accountData?.payment?.phone),
      payerZip: accountData?.payment?.zip || '',
    },
  })

  const isPayerAddressDifferent = form.watch('payerAddressIfDifferent')

  async function onSubmit(values: z.infer<typeof schema>) {
    const deliveryType = pathname === '/checkout' ? 'pickup' : 'haypost'
    const postValues = cleanObject({
      email: values.email,

      shipping: {
        name: values.firstName,
        lastname: values.lastName,
        address_1: values.address,
        phone: values.phone && `+374${values.phone}`,
        city: values.cityOrVillage,
        state: values.state,
        zip: values.zip,
        same_for_payment: values.payerAddressIfDifferent ? 0 : 1,
      },
      payment: {
        name: values.payerFirstName,
        lastname: values.payerLastName,
        address_1: values.payerAddress,
        phone: values.payerPhone && `+374${values.payerPhone}`,
        city: values.payerCityOrVillage,
        state: values.payerState,
        zip: values.payerZip,
      },

      // profile
      name: isInPayment ? undefined : values.profileFirstName,
      lastname: isInPayment ? undefined : values.profileLastName,
      phone: isInPayment
        ? undefined
        : values.profilePhone && `+374${values.profilePhone}`,
      subscribe: isInPayment ? undefined : values.subscribe,

      // payment
      basket_products: isInPayment
        ? checkoutData?.products.map((item) => item.id)
        : undefined,
      delivery_type: isInPayment ? deliveryType : undefined,
      comment: isInPayment ? values.note : undefined,
      payment_method_id: isInPayment ? paymentType : undefined,
      shipping_area_id: currentShipping?.shipping_area_id,
      gift_card_id: gift_card,
      promotion_id: promotion,
    })
    // console.log('🚀 ~ file: ProfileAndPay.tsx:165 ~ postValues:', postValues)
    mutation.mutate(postValues)
  }

  const resetPayerFields = (checked: boolean) => {
    if (checked) {
      form.setValue('payerFirstName', '')
      form.setValue('payerLastName', '')
      form.setValue('payerAddress', '')
      form.setValue('payerCityOrVillage', '')
      form.setValue('payerState', '')
      form.setValue('payerPhone', '')
    }
  }

  return (
    <>
      <Form {...form}>
        <form
          onSubmit={form.handleSubmit(onSubmit)}
          className='flex flex-col justify-center gap-4 p-4 sm:px-0'
        >
          {!isInPayment && <h2 className='text-lg'>Օգտատիրոջ տվյալներ</h2>}
          <FormField
            name='email'
            render={({ field }) => (
              <FormItem className='space-y-1'>
                <FormLabel required={true}>էլ․ հասցե</FormLabel>
                <FormControl>
                  <Input
                    {...field}
                    disabled={!isInPayment}
                    onBlur={(e) => {
                      setCheckoutStore('promotion', '')
                      setCheckoutStore('email', e.target.value)
                    }}
                    type='email'
                  />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          {!isInPayment && (
            <>
              {!isInPayment && (
                <h2 className='text-lg'>Կոնտակտային տվյալներ</h2>
              )}

              <div className='flex gap-4 sm:flex-col'>
                <FormField
                  name='profileFirstName'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Անուն</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='profileLastName'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Ազգանուն</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <div className='relative w-full max-w-44 sm:max-w-full'>
                  <FormField
                    name='profilePhone'
                    render={({ field }) => (
                      <FormItem className='space-y-1'>
                        <FormLabel required={true}>Հեռախոս</FormLabel>
                        <FormControl>
                          <Input
                            {...field}
                            className='pl-14'
                            inputType='integer'
                            maxLength={8}
                            onChange={(e) =>
                              field.onChange(
                                formatInputValue(e.target.value, 'integer')
                              )
                            }
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  <div className='absolute left-3 top-9'>+374</div>
                </div>
              </div>
            </>
          )}
          {!isInPayment && <h2 className='text-lg'>Առաքման հասցե</h2>}
          <div className='flex gap-4 sm:flex-col'>
            <FormField
              name='firstName'
              render={({ field }) => (
                <FormItem className='space-y-1'>
                  <FormLabel required={true}>Անուն</FormLabel>
                  <FormControl>
                    <Input {...field} />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
            <FormField
              name='lastName'
              render={({ field }) => (
                <FormItem className='space-y-1'>
                  <FormLabel required={true}>Ազգանուն</FormLabel>
                  <FormControl>
                    <Input {...field} />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
          </div>
          <FormField
            name='address'
            render={({ field }) => (
              <FormItem className='space-y-1'>
                <FormLabel required={true}>Առաքման Հասցե</FormLabel>
                <FormControl>
                  <Input {...field} />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <div className='flex gap-4 lg:flex-col'>
            <div className='flex w-full gap-4 sm:flex-col'>
              <FormField
                name='cityOrVillage'
                render={({ field }) => (
                  <FormItem className='space-y-1'>
                    <FormLabel required={true}>Քաղաք/գյուղ</FormLabel>
                    <FormControl>
                      <Input {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                name='state'
                render={({ field }) => (
                  <FormItem className='max-w-44 space-y-1 lg:max-w-full'>
                    <FormLabel required={true}>Մարզ</FormLabel>
                    <FormControl>
                      <Select
                        onValueChange={(value) => {
                          field.onChange(value)
                          if (!isPayerAddressDifferent) {
                            setCheckoutStore('state_1', value)
                          }
                        }}
                        defaultValue={field.value}
                      >
                        <SelectTrigger className='w-full'>
                          <SelectValue placeholder='Ընտրել մարզը' />
                        </SelectTrigger>
                        <SelectContent>
                          <SelectGroup>
                            {states.map((state) => (
                              <SelectItem
                                key={state.shortName}
                                value={state.shortName}
                                className='py-1'
                              >
                                {state.name}
                              </SelectItem>
                            ))}
                          </SelectGroup>
                        </SelectContent>
                      </Select>
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
            </div>

            <div className='flex gap-4 sm:flex-col'>
              <div className='relative w-44 shrink-0 lg:max-w-full'>
                <FormField
                  name='phone'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Հեռախոս</FormLabel>
                      <FormControl>
                        <Input
                          {...field}
                          className='pl-14'
                          inputType='integer'
                          maxLength={8}
                          onChange={(e) =>
                            field.onChange(
                              formatInputValue(e.target.value, 'integer')
                            )
                          }
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <div className='absolute left-3 top-9'>+374</div>
              </div>
              <FormField
                name='zip'
                render={({ field }) => (
                  <FormItem className='w-44 shrink-0 space-y-1 lg:max-w-full'>
                    <FormLabel required={true}>Փոստային ինդեքս</FormLabel>
                    <FormControl>
                      <Input
                        {...field}
                        inputType='integer'
                        maxLength={4}
                        onChange={(e) => {
                          field.onChange(
                            formatInputValue(e.target.value, 'integer')
                          )
                        }}
                      />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
            </div>
          </div>
          {isInPayment && (
            <FormField
              name='note'
              render={({ field }) => (
                <FormItem className='space-y-1'>
                  <FormLabel>
                    Նշումներ
                    <span className='text-muted-foreground'>
                      (առաքման նախընտրելի օր, ժամ և այլն)
                    </span>
                  </FormLabel>
                  <FormControl>
                    <Textarea {...field} />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
          )}
          <div className={cn('text-xl', isInPayment ? 'pt-2' : 'pt-4')}>
            Վճարողի հասցե (եթե տարբերվում է առաքման հասցեից)
          </div>
          <FormField
            control={form.control}
            name='payerAddressIfDifferent'
            render={({ field }) => (
              <FormItem className='flex items-center justify-between space-y-0'>
                <FormLabel className='w-full p-2 pl-0'>
                  Առաքման հասցեն տարբերվում է հաճախորդի հասցեից (լրացրեք եթե
                  դուք չեք ստանալու պատվերը)
                </FormLabel>
                <FormControl>
                  <Checkbox
                    checked={field.value}
                    onCheckedChange={(checked) => {
                      field.onChange(checked)
                      resetPayerFields(Boolean(checked))
                      if (!checked) {
                        setCheckoutStore('another_shipping_address', '0')
                      }
                    }}
                  />
                </FormControl>
              </FormItem>
            )}
          />

          {/* Payer */}
          {isPayerAddressDifferent && (
            <>
              <div className='flex gap-4 sm:flex-col'>
                <FormField
                  name='payerFirstName'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Անուն</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='payerLastName'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Ազգանուն</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
              </div>
              <FormField
                name='payerAddress'
                render={({ field }) => (
                  <FormItem className='space-y-1'>
                    <FormLabel required={true}>Առաքման Հասցե</FormLabel>
                    <FormControl>
                      <Input {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <div className='flex gap-4 sm:flex-col'>
                <FormField
                  name='payerCityOrVillage'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Քաղաք/գյուղ</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='payerState'
                  render={({ field }) => (
                    <FormItem className='space-y-1'>
                      <FormLabel required={true}>Մարզ</FormLabel>
                      <FormControl>
                        <Select
                          onValueChange={(value) => {
                            field.onChange(value)
                            if (isPayerAddressDifferent) {
                              setCheckoutStore('state_2', value)
                            }
                          }}
                          defaultValue={field.value}
                        >
                          <SelectTrigger className='w-full'>
                            <SelectValue placeholder='Ընտրել մարզը' />
                          </SelectTrigger>
                          <SelectContent>
                            <SelectGroup>
                              {states.map((state) => (
                                <SelectItem
                                  key={state.shortName}
                                  value={state.shortName}
                                  className='py-1'
                                >
                                  {state.name}
                                </SelectItem>
                              ))}
                            </SelectGroup>
                          </SelectContent>
                        </Select>
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <div className='relative w-full'>
                  <FormField
                    name='payerPhone'
                    render={({ field }) => (
                      <FormItem className='space-y-1'>
                        <FormLabel required={true}>Հեռախոս</FormLabel>
                        <FormControl>
                          <Input
                            {...field}
                            className='pl-14'
                            inputType='integer'
                            maxLength={8}
                            onChange={(e) =>
                              field.onChange(
                                formatInputValue(e.target.value, 'integer')
                              )
                            }
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  <div className='absolute left-3 top-9'>+374</div>
                </div>
                <FormField
                  name='payerZip'
                  render={({ field }) => (
                    <FormItem className='w-44 shrink-0 space-y-1 lg:max-w-full'>
                      <FormLabel>Փոստային ինդեքս</FormLabel>
                      <FormControl>
                        <Input
                          {...field}
                          inputType='integer'
                          maxLength={4}
                          onChange={(e) => {
                            field.onChange(
                              formatInputValue(e.target.value, 'integer')
                            )
                          }}
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
              </div>
            </>
          )}

          {isInPayment && (
            <>
              <PaymentMethods
                paymentType={paymentType}
                setPaymentType={setPaymentType}
              />
              <Button
                type='submit'
                className='font-bold'
                disabled={mutation.isPending}
              >
                ԱՎԱՐՏԵԼ ՊԱՏՎԵՐԸ ({convertToMoney(checkoutData?.total)})
              </Button>
            </>
          )}
          {!isInPayment && (
            <>
              {!isInPayment && (
                <h2 className='pt-4 text-lg'>
                  Բաժանորդագրվել մեր նորություններին
                </h2>
              )}
              <div className='flex items-center'>
                <FormField
                  control={form.control}
                  name='subscribe'
                  render={({ field }) => (
                    <FormItem className='w-auto'>
                      <FormControl>
                        <Checkbox
                          id='subscribe'
                          checked={field.value}
                          onCheckedChange={field.onChange}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />
                <Label
                  htmlFor='subscribe'
                  className='w-full pl-2 text-sm font-medium peer-disabled:cursor-not-allowed peer-disabled:opacity-70'
                >
                  Նորություններ և հատուկ առաջարկներ բաժանորդների համար
                </Label>
              </div>
              <div className='flex gap-4 self-end pt-0'>
                <Button type='submit' disabled={mutation.isPending}>
                  Պահպանել
                </Button>
              </div>
            </>
          )}
        </form>
      </Form>
    </>
  )
}

export default ProfileAndPay
