'use client'

import { useState } from 'react'

import { useGoogleLogin } from '@react-oauth/google'
import { useQuery } from '@tanstack/react-query'
import axios, { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import { FaFacebookF } from 'react-icons/fa6'
import { FcGoogle } from 'react-icons/fc'

import { cn } from '@/lib/utils'
import { getCart } from '@/server/getCart'
import axiosClient from '@/services/axiosClient'
import { setNextCookies } from '@/utils/setNextCookies'

import { fbLogin } from './FacebookSDK'
import { Button } from './ui/button'
import { useToast } from './ui/use-toast'

export default function SocialLogin({
  borderPrimary,
}: {
  borderPrimary?: boolean
}) {
  const [loading, setLoading] = useState(false)
  const { toast } = useToast()
  const { data: cartData } = useQuery({
    queryKey: ['cart'],
    queryFn: getCart,
  })

  async function handleGoogleLogin(
    name: string,
    lastname: string,
    email: string
  ) {
    setLoading(true)
    try {
      await axiosClient('sanctum/csrf-cookie')
      const res = await axiosClient.post('api/social-login', {
        userData: {
          name,
          lastname,
          email,
        },
        provider: 'google',
      })
      const token = res.data.token
      if (cartData && cartData?.products?.length > 0) {
        await axios.post(
          `${process.env.NEXT_PUBLIC_API_URL}api/carts`,
          {
            products: cartData.products?.map((item) => ({
              haysell_id: item.product.haysell_id,
              quantity: item.quantity,
              variation_id: item.variation?.id,
            })),
            is_cart: true,
          },
          {
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        )
      }
      Cookies.set('siamoods_token', token)
      setNextCookies('siamoods_token', token)
      window.location.href = '/'
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  const googleLogin = useGoogleLogin({
    onSuccess: async (tokenResponse) => {
      const userInfoResponse = await fetch(
        'https://www.googleapis.com/oauth2/v1/userinfo?alt=json',
        {
          headers: {
            Authorization: `Bearer ${tokenResponse.access_token}`,
          },
        }
      )

      const userInfo = await userInfoResponse.json()
      const { email, given_name, family_name } = userInfo
      handleGoogleLogin(given_name, family_name, email)
    },
    onError: () => {
      console.log('Login Failed')
    },
  })

  const handleFacebookLogin = async (
    name: string,
    lastname: string,
    email: string
    // accessToken: string
  ) => {
    setLoading(true)
    try {
      await axiosClient('sanctum/csrf-cookie')
      const res = await axiosClient.post('api/social-login', {
        userData: {
          name,
          lastname,
          email,
        },
        provider: 'facebook',
      })
      const token = res.data.token
      if (cartData && cartData?.products?.length > 0) {
        await axios.post(
          `${process.env.NEXT_PUBLIC_API_URL}api/carts`,
          {
            products: cartData.products?.map((item) => ({
              haysell_id: item.product.haysell_id,
              quantity: item.quantity,
              variation_id: item.variation?.id,
            })),
            is_cart: true,
          },
          {
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        )
      }
      Cookies.set('siamoods_token', token)
      setNextCookies('siamoods_token', token)
      window.location.href = '/'
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  function facebookLogin() {
    fbLogin().then((response: any) => {
      if (response.status === 'connected') {
        // const accessToken = response.authResponse.accessToken
        window.FB.api(
          '/me',
          { fields: 'name,email,first_name,last_name' },
          (userInfo: any) => {
            const { email, first_name, last_name } = userInfo
            handleFacebookLogin(first_name, last_name, email)
            // handleFacebookLogin(first_name, last_name, email, accessToken)
          }
        )
      } else {
        // something
      }
    })
  }

  return (
    <div className='flex w-full flex-col items-center gap-2'>
      <p className='text-muted-foreground'>
        Մուտք գործել՝ օգտագործելով սոց. ցանց:
      </p>
      <div className='grid w-full grid-cols-2 gap-4'>
        <Button
          type='button'
          className={cn(
            'flex-1 text-2xl text-yellow-500',
            borderPrimary && 'border-primary'
          )}
          variant='outline'
          disabled={loading}
          onClick={() => googleLogin()}
        >
          <FcGoogle />
        </Button>

        <Button
          type='button'
          className={cn(
            'flex-1 text-2xl !text-blue-500',
            borderPrimary && 'border-primary'
          )}
          variant='outline'
          disabled={loading}
          onClick={facebookLogin}
        >
          <FaFacebookF />
        </Button>
      </div>
    </div>
  )
}
