'use client'

import { useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import { AxiosError } from 'axios'
import ReCAPTCHA from 'react-google-recaptcha'
// import ReCAPTCHA from 'react-google-recaptcha'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import { Button } from '@/components/ui/button'
import {
  Dialog,
  DialogContent,
  DialogHeader,
  DialogTrigger,
} from '@/components/ui/dialog'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { StarRatings } from '@/components/ui/starRating'
import { Textarea } from '@/components/ui/textarea'
import { useToast } from '@/components/ui/use-toast'
import axiosClient from '@/services/axiosClient'

const formSchema = z.object({
  name: z.string().min(1, { message: 'Անունը պարտադիր է' }),
  rating: z.string().min(1, { message: 'Գնահատականը պարտադիր է' }),
  review: z.string().min(1, { message: 'Կարծիքը պարտադիր է' }),
})

const AddReviewDialog = ({
  haysell_id,
}: {
  haysell_id: number | undefined
}) => {
  const { toast } = useToast()
  const [loading, setLoading] = useState(false)
  const [captchaToken, setCaptchaToken] = useState<string | null>(null)
  const [isDialogOpen, setIsDialogOpen] = useState(false)
  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      name: '',
      rating: '',
      review: '',
    },
  })

  async function onSubmit(values: z.infer<typeof formSchema>) {
    if (!haysell_id && haysell_id !== 0) return

    setLoading(true)
    try {
      await axiosClient.post('api/save-review', {
        haysell_id,
        captchaToken,
        ...values,
      })
      toast({
        title: 'Կարծիքը ավելացված է',
      })
      // close dialog after success
      const event = new KeyboardEvent('keydown', {
        key: 'Escape',
      })
      document.dispatchEvent(event)
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  return (
    <Dialog
      open={isDialogOpen}
      onOpenChange={(open) => {
        form.reset()
        setIsDialogOpen(open)
      }}
      modal={false}
    >
      <DialogTrigger asChild>
        <Button className='self-start' variant='outline'>
          Թողնել Կարծիք
        </Button>
      </DialogTrigger>
      {isDialogOpen && (
        <div
          className='fixed inset-0 z-20 bg-black/50'
          onClick={() => setIsDialogOpen(false)}
        />
      )}
      <DialogContent
        className='p-0'
        onInteractOutside={(e) => {
          e.preventDefault()
        }}
      >
        <Form {...form}>
          <form onSubmit={form.handleSubmit(onSubmit)}>
            <DialogHeader className='bg-muted p-4 text-start text-muted-foreground'>
              <h2>Թողնել կարծիք</h2>
            </DialogHeader>
            <div className='flex flex-col gap-6 p-4'>
              <FormField
                name='name'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Ձեր Անունը</FormLabel>
                    <FormControl>
                      <Input {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                name='rating'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Գնահատական</FormLabel>
                    <FormControl>
                      <StarRatings
                        variant='primary'
                        rating={field.value}
                        onRatingChange={(rating) => {
                          field.onChange(String(rating))
                        }}
                        {...field}
                      />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                name='review'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Թողնել կարծիք</FormLabel>
                    <FormControl>
                      <Textarea {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <ReCAPTCHA
                sitekey={process.env.NEXT_PUBLIC_RE_CAPTCHA_KEY as string}
                onChange={(token) => setCaptchaToken(token)}
              />
            </div>
            <div className='flex justify-end space-y-4 border-t p-4'>
              <Button disabled={loading}>Հաստատել</Button>
            </div>
          </form>
        </Form>
      </DialogContent>
    </Dialog>
  )
}

export default AddReviewDialog
